<?php

namespace Drupal\route_in_modal;

use Drupal;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Ajax\PrependCommand;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Ajax\RemoveCommand;
use Drupal\Core\Form\FormStateInterface;

/**
 * A helper class for creating Ajax responses for Route In Modal.
 */
class RouteInModalAjaxHelper {

  /**
   * @param array $form
   * @param Drupal\Core\Form\FormStateInterface $formState
   *
   * @return Drupal\Core\Ajax\AjaxResponse
   */
  public static function ajaxCallback(array $form, FormStateInterface $formState): AjaxResponse {
    $response = new AjaxResponse();

    $messages = Drupal::messenger()->all();
    $config = Drupal::config('route_in_modal.settings');

    if (!isset($messages['error'])) {
      $response->addCommand(new CloseModalDialogCommand());

      $confirmationMessagesModal = $config->get('confirmation_messages_modal');

      if ($confirmationMessagesModal && count($messages) > 0) {
        Drupal::messenger()->deleteAll();

        $display = strip_tags($messages['status'][0]->__toString());

        $dialogWidth = $config->get('dialog_width');
        if (empty($dialogWidth) && $dialogWidth !== '0') {
          $dialogWidth = '480';
        }

        $dialogHeight = $config->get('dialog_height');
        if (empty($dialogHeight) && $dialogHeight !== '0') {
          $dialogHeight = 'auto';
        }

        $options = [
          'width' => $dialogWidth,
          'height' => $dialogHeight,
          'dialogClass' => 'views-ui-dialog js-views-ui-dialog',
        ];

        $response->addCommand(new OpenModalDialogCommand('', $display, $options));
      }

      $response->addCommand(self::redirectCommand($formState));
    } else {
      /** @var Drupal\Core\Render\RendererInterface $renderer */
      $renderer = Drupal::service('renderer');
      $messagesElement = [
        '#type' => 'container',
        '#attributes' => [
          'class' => 'route-in-modal-messages',
        ],
        'messages' => ['#type' => 'status_messages'],
      ];

      $response->addCommand(new RemoveCommand('.route-in-modal-messages'));

      $response->addCommand(new PrependCommand(
        '#route_in_modal_wrapper',
        $renderer->renderRoot($messagesElement)
      ));
    }

    return $response;
  }

  /**
   * @param Drupal\Core\Form\FormStateInterface $formState
   *
   * @return Drupal\Core\Ajax\RedirectCommand
   */
  public static function redirectCommand(FormStateInterface $formState): RedirectCommand {
    return new RedirectCommand(Drupal::request()->headers->get('referer') . $formState->getRedirect());
  }
}
